@@ -35,33 +35,88 @@ namespace osl
 
       struct OpenMidEndingEvalDebugInfo
       {
-	enum
-	{
-	  EVAL, OPENING, ENDGAME, PROGRESS_INDEPENDENT,
-	  PROGRESS,
-	  // progress independent
-	  PIECE, ROOK_MOBILITY, BISHOP_MOBILITY, LANCE_MOBILITY,
-	  PIECE_PAIR, KING25_EFFECT,
-	  // opening and ending
-	  KING_PIECE_OPENING, KING_PIECE_ENDING,
-	  PIECE_STAND_OPENING, PIECE_STAND_ENDING,
-	  PAWN_DROP_OPENING, PAWN_DROP_ENDING,
-	  PTYPE_X_OPENING, PTYPE_X_ENDING,
-	  PTYPE_Y_OPENING, PTYPE_Y_ENDING,
-	  KING25_EACH_OPENING, KING25_EACH_ENDING,
-	  NO_PAWN_OPENING, NO_PAWN_ENDING,
-	  GOLD_RETREAT_OPENING, GOLD_RETREAT_ENDING,
-	  SILVER_RETREAT_OPENING, SILVER_RETREAT_ENDING,
-	  KNIGHT_BLOCKED_OPENING, KNIGHT_BLOCKED_ENDING,
-	  KINGX_BLOCKED_OPENING, KINGX_BLOCKED_ENDING,
-	  ROOK_PAWN_OPENING, ROOK_PAWN_ENDING,
-	  PIN_OPENING, PIN_ENDING,
-	  ALL_GOLD_OPENING, ALL_GOLD_ENDING,
-	  ANAGUMA_EMPTY_OPENING, ANAGUMA_EMPTY_ENDING,
-	  STAND_EXCEPT_PAWN_OPENING, STAND_EXCEPT_PAWN_ENDING,
-	  FEATURE_LIMIT
-	};
-	CArray<int, FEATURE_LIMIT> values;
+        enum StageFeature
+        {
+          KING_PIECE_RELATIVE,
+          PIECE_STAND,
+          KING25_EFFECT_EACH,
+          PTYPEX,
+          PTYPEY,
+          ROOK_MOBILITY,
+          BISHOP_MOBILITY,
+          LANCE_MOBILITY,
+          ROOK_EFFECT,
+          BISHOP_EFFECT,
+          PIECE_STAND_COMBINATION,
+          PIECE_STAND_TURN,
+          ROOK_PAWN,
+          PAWN_DROP,
+          PIECE_STAND_Y,
+          KNIGHT_CHECK,
+          PAWN_ADVANCE,
+          PAWN_PTYPEO,
+          PROMOTED_MINOR_PIECE,
+          KING_PIECE_RELATIVE_NOSUPPORT,
+          NON_PAWN_ATTACKED,
+          NON_PAWN_ATTACKED_PTYPE,
+          PTYPE_YY,
+          KING3PIECES,
+          BISHOP_HEAD,
+          KNIGHT_HEAD,
+          ROOK_PROMOTE_DEFENSE,
+          PTYPE_COUNT,
+          LANCE_EFFECT_PIECE,
+          PTYPE_Y_PAWN_Y,
+          BISHOP_AND_KING,
+          PIECE_FORK_TURN,
+          ROOK_SILVER_KNIGHT,
+          BISHOP_SILVER_KNIGHT,
+          KING25_EFFECT_SUPPORTED,
+          KING_ROOK_BISHOP,
+          KING_X_BLOCKED3,
+          GOLD_RETREAT,
+          SILVER_RETREAT,
+          ALL_GOLD,
+          ALL_MAJOR,
+          KING25_EFFECT_DEFENSE,
+          ANAGUMA_EMPTY,
+          NO_PAWN_ON_STAND,
+          NON_PAWN_PIECE_STAND,
+          PIN_PTYPE_ALL,
+          KING_MOBILITY,
+          GOLD_AND_SILVER_NEAR_KING,
+          PTYPE_COMBINATION,
+          KING25_BOTH_SIDE,
+          KING25_MOBILITY,
+          BISHOP_STAND_FILE5,
+          MAJOR_CHECK_WITH_CAPTURE,
+          SILVER_ADVANCE26,
+          KING25_EFFECT3,
+          BISHOP_BISHOP_PIECE,
+          ROOK_ROOK,
+          ROOK_ROOK_PIECE,
+          KING25_EFFECT_COUNT_COMBINATION,
+          NON_PAWN_ATTACKED_PTYPE_PAIR,
+          ATTACK_MAJORS_IN_BASE,
+          STAGE_FEATURE_LIMIT
+        };
+        enum ProgressIndependentFeature
+        {
+          PIECE,
+          BISHOP_EXCHANGE_SILVER_KING, // recalculated_value
+          ENTER_KING_DEFENSE,
+          KING25_EFFECT_ATTACK, // end recalculated_value
+          PIECE_PAIR,
+          PIECE_PAIR_KING,
+          PROGRESS_INDEPENDENT_FEATURE_LIMIT
+        };
+        int value;
+        int progress;
+        CArray<int, PROGRESS_INDEPENDENT_FEATURE_LIMIT> progress_independent_values;
+        CArray<MultiInt, STAGE_FEATURE_LIMIT> stage_values;
+
+	static const char *name(ProgressIndependentFeature);
+	static const char *name(StageFeature);
       };
 
       class OpenMidEndingEval
