@@ -113,7 +113,8 @@ namespace osl
       /**
        * searchWithSecondsForThisMove を呼び出す
        */
-      const MoveWithComment selectBestMove(const GameState&, int, int, int);
+      const MoveWithComment selectBestMove(const GameState&, int limit, int elapsed,
+                                           int byoyomi);
       const MoveWithComment selectBestMoveInTime(const GameState&, const search::TimeAssigned&);
       static const search::TimeAssigned assignTime(const GameState& state, int limit, int elapsed,
 						   int byoyomi, int verbose);
