@@ -13,6 +13,24 @@ namespace osl
   {
     namespace ml
     {
+      struct NewProgressDebugInfo
+      {
+        enum Feature
+        {
+          ATTACK_5X3,
+          DEFENSE_5X3,
+          ATTACK5X5,
+          STAND,
+          EFFECT5X5,
+          KING_RELATIVE_ATTACK,
+          KING_RELATIVE_DEFENSE,
+          NON_PAWN_ATTACKED_PAIR,
+          FEATURE_LIMIT
+        };
+        CArray<int, FEATURE_LIMIT> black_values;
+        CArray<int, FEATURE_LIMIT> white_values;
+      };
+
       struct NewProgressData
       {
 	CArray<MultiInt,2> non_pawn_ptype_attacked_pair_eval;
@@ -189,6 +207,7 @@ namespace osl
 	  else
 	    updateSub<BLACK>(new_state,last_move);
 	}
+        NewProgressDebugInfo debugInfo() const;
       private:
 	template<Player P>
 	void updateMain(const NumEffectState &new_state, Move last_move);
