@@ -14,16 +14,18 @@ namespace osl
       virtual ~SearchMonitor();
       
       virtual void newDepth(int depth);
-      virtual void showPV(int depth, size_t node_count, double elapsed, int value, Move cur, const Move *first, const Move *last);
+      virtual void showPV(int depth, size_t node_count, double elapsed, int value, Move cur, const Move *first, const Move *last, 
+			  const bool *threatmate_first, const bool *threatmate_last);
       virtual void rootMove(Move cur);
       virtual void rootFirstMove(Move cur);
       virtual void timeInfo(size_t node_count, double elapsed);
       virtual void hashInfo(double ratio);
       virtual void rootForcedMove(Move the_move);
       virtual void rootLossByCheckmate();
+      virtual void depthFinishedNormally(int depth);
     };
-
   }
+  using search::SearchMonitor;
 }
 
 #endif /* OSL_SEARCHMONITOR_H */
