@@ -30,6 +30,89 @@ OpenMidEndingPtypeTable()
     }};
   reset(values);
 }
+#ifndef MINIMAL
+const char * osl::eval::ml::
+OpenMidEndingEvalDebugInfo::name(ProgressIndependentFeature f)
+{
+  static const CArray<const char *,PROGRESS_INDEPENDENT_FEATURE_LIMIT> table = {{
+      "PIECE",
+      "BISHOP_EXCHANGE_SILVER_KING",
+      "ENTER_KING_DEFENSE",
+      "KING25_EFFECT_ATTACK",
+      "PIECE_PAIR",
+      "PIECE_PAIR_KING",
+    }};
+  return table[f];
+}
+const char *osl::eval::ml::
+OpenMidEndingEvalDebugInfo::name(StageFeature f)
+{
+  static const CArray<const char *,STAGE_FEATURE_LIMIT> table = {{
+          "KING_PIECE_RELATIVE",
+          "PIECE_STAND",
+          "KING25_EFFECT_EACH",
+          "PTYPEX",
+          "PTYPEY",
+          "ROOK_MOBILITY",
+          "BISHOP_MOBILITY",
+          "LANCE_MOBILITY",
+          "ROOK_EFFECT",
+          "BISHOP_EFFECT",
+          "PIECE_STAND_COMBINATION",
+          "PIECE_STAND_TURN",
+          "ROOK_PAWN",
+          "PAWN_DROP",
+          "PIECE_STAND_Y",
+          "KNIGHT_CHECK",
+          "PAWN_ADVANCE",
+          "PAWN_PTYPEO",
+          "PROMOTED_MINOR_PIECE",
+          "KING_PIECE_RELATIVE_NOSUPPORT",
+          "NON_PAWN_ATTACKED",
+          "NON_PAWN_ATTACKED_PTYPE",
+          "PTYPE_YY",
+          "KING3PIECES",
+          "BISHOP_HEAD",
+          "KNIGHT_HEAD",
+          "ROOK_PROMOTE_DEFENSE",
+          "PTYPE_COUNT",
+          "LANCE_EFFECT_PIECE",
+          "PTYPE_Y_PAWN_Y",
+          "BISHOP_AND_KING",
+          "PIECE_FORK_TURN",
+          "ROOK_SILVER_KNIGHT",
+          "BISHOP_SILVER_KNIGHT",
+          "KING25_EFFECT_SUPPORTED",
+          "KING_ROOK_BISHOP",
+          "KING_X_BLOCKED3",
+          "GOLD_RETREAT",
+          "SILVER_RETREAT",
+          "ALL_GOLD",
+          "ALL_MAJOR",
+          "KING25_EFFECT_DEFENSE",
+          "ANAGUMA_EMPTY",
+          "NO_PAWN_ON_STAND",
+          "NON_PAWN_PIECE_STAND",
+          "PIN_PTYPE_ALL",
+          "KING_MOBILITY",
+          "GOLD_AND_SILVER_NEAR_KING",
+          "PTYPE_COMBINATION",
+          "KING25_BOTH_SIDE",
+          "KING25_MOBILITY",
+          "BISHOP_STAND_FILE5",
+          "MAJOR_CHECK_WITH_CAPTURE",
+          "SILVER_ADVANCE26",
+          "KING25_EFFECT3",
+          "BISHOP_BISHOP_PIECE",
+          "ROOK_ROOK",
+          "ROOK_ROOK_PIECE",
+          "KING25_EFFECT_COUNT_COMBINATION",
+          "NON_PAWN_ATTACKED_PTYPE_PAIR",
+          "ATTACK_MAJORS_IN_BASE",
+    }};
+  return table[f];
+}
+#endif
 
 volatile osl::eval::ml::OpenMidEndingEval::LoadStatus
 osl::eval::ml::OpenMidEndingEval::initialized_flag = osl::eval::ml::OpenMidEndingEval::Zero;
@@ -1033,15 +1116,13 @@ osl::eval::ml::OpenMidEndingEvalDebugInfo
 osl::eval::ml::OpenMidEndingEval::debugInfo(const NumEffectState &state)
 {
   OpenMidEndingEvalDebugInfo debug_info;
-  debug_info.values[OpenMidEndingEvalDebugInfo::EVAL] = value();
-  debug_info.values[OpenMidEndingEvalDebugInfo::OPENING] = openingValue();
-  debug_info.values[OpenMidEndingEvalDebugInfo::ENDGAME] = endgameValue();
-  debug_info.values[OpenMidEndingEvalDebugInfo::PROGRESS_INDEPENDENT] = progressIndependentValue();
-  debug_info.values[OpenMidEndingEvalDebugInfo::PROGRESS] = progress16().value();
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIECE] = progress_independent_value;
-  debug_info.values[OpenMidEndingEvalDebugInfo::ROOK_MOBILITY] = 0; // XXX progress dependent now
-  debug_info.values[OpenMidEndingEvalDebugInfo::BISHOP_MOBILITY] = 0; // XXX
-  debug_info.values[OpenMidEndingEvalDebugInfo::LANCE_MOBILITY] = 0; // XXX
+  debug_info.value = value();
+  debug_info.progress = progress16().value();
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::PIECE] = progress_independent_value;
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::BISHOP_EXCHANGE_SILVER_KING]
+    = BishopExchangeSilverKing::eval(state);
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::ENTER_KING_DEFENSE]
+    = EnterKingDefense::eval(state);
   int black_attack_effect, black_attack_piece, black_defense_effect, black_defense_piece,
     white_attack_effect, white_attack_piece, white_defense_effect, white_defense_piece;
   CArray<int, 5> black_vertical, white_vertical,
@@ -1060,7 +1141,7 @@ osl::eval::ml::OpenMidEndingEval::debugInfo(const NumEffectState &state)
 						    white_attack_supported_piece,
 						    black_vertical,
 						    black_king_vertical);
-  debug_info.values[OpenMidEndingEvalDebugInfo::KING25_EFFECT] =
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT_ATTACK] =
     King25EffectBoth::eval(state,
 			   black_attack_effect,
 			   black_attack_piece,
@@ -1073,55 +1154,118 @@ osl::eval::ml::OpenMidEndingEval::debugInfo(const NumEffectState &state)
 			white_attack_effect, white_attack_piece,
 			black_defense_effect, black_defense_piece,
 			white_defense_effect, white_defense_piece);
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIECE_PAIR] = piece_pair_value;
-  debug_info.values[OpenMidEndingEvalDebugInfo::KING_PIECE_OPENING] = king_table_value[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KING_PIECE_ENDING] = king_table_value[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIECE_STAND_OPENING] = piece_stand_value[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIECE_STAND_ENDING] = piece_stand_value[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KING25_EACH_OPENING] = king25_effect_each[BLACK][0] + king25_effect_each[WHITE][0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KING25_EACH_ENDING] = king25_effect_each[WHITE][EndgameIndex] + king25_effect_each[WHITE][EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PAWN_DROP_ENDING] = pawn_drop[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PAWN_DROP_ENDING] = pawn_drop[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PTYPE_X_OPENING] = ptypex[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PTYPE_X_ENDING] = ptypex[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PTYPE_Y_OPENING] = ptypey[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PTYPE_Y_ENDING] = ptypey[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::NO_PAWN_OPENING] =
-    NoPawnOnStand::eval(state, black_pawn_count)[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::NO_PAWN_ENDING] =
-    NoPawnOnStand::eval(state, black_pawn_count)[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::GOLD_RETREAT_OPENING] =
-    GoldRetreat::eval(state)[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::GOLD_RETREAT_ENDING] =
-    GoldRetreat::eval(state)[EndgameIndex];
-//   debug_info.values[OpenMidEndingEvalDebugInfo::SILVER_RETREAT_OPENING] =
-//     SilverRetreat::eval(state)[0];
-//   debug_info.values[OpenMidEndingEvalDebugInfo::SILVER_RETREAT_ENDING] =
-//     SilverRetreat::eval(state)[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KNIGHT_BLOCKED_OPENING] =
-    knight_advance[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KNIGHT_BLOCKED_ENDING] =
-    knight_advance[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KINGX_BLOCKED_OPENING] =
-    kingx_blocked[BLACK][0] + kingx_blocked[WHITE][0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::KINGX_BLOCKED_ENDING] =
-    kingx_blocked[BLACK][EndgameIndex] + kingx_blocked[WHITE][EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIN_OPENING] =
-    Pin::eval(state, state.pin(BLACK), state.pin(WHITE))[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::PIN_ENDING] =
-    Pin::eval(state, state.pin(BLACK), state.pin(WHITE))[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::ALL_GOLD_OPENING] =
-    AllGold::eval(black_gold_count)[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::ALL_GOLD_ENDING] =
-    AllGold::eval(black_gold_count)[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::ANAGUMA_EMPTY_OPENING] =
-    AnagumaEmpty::eval(state)[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::ANAGUMA_EMPTY_ENDING] =
-    AnagumaEmpty::eval(state)[EndgameIndex];
-  debug_info.values[OpenMidEndingEvalDebugInfo::STAND_EXCEPT_PAWN_OPENING] =
-    NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE])[0];
-  debug_info.values[OpenMidEndingEvalDebugInfo::STAND_EXCEPT_PAWN_ENDING] =
-    NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE])[EndgameIndex];
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::PIECE_PAIR] = piece_pair_value;
+  debug_info.progress_independent_values[OpenMidEndingEvalDebugInfo::PIECE_PAIR_KING] = piece_pair_king_value[BLACK] + piece_pair_king_value[WHITE];
+
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING_PIECE_RELATIVE] =
+    king_table_value;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIECE_STAND] =
+    piece_stand_value;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT_EACH] =
+    king25_effect_each[BLACK] + king25_effect_each[WHITE];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPEX] = ptypex;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPEY] = ptypey;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_MOBILITY] = rook_mobility;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_MOBILITY] = bishop_mobility;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::LANCE_MOBILITY] = lance_mobility;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_EFFECT] = rook_effect;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_EFFECT] = bishop_effect;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIECE_STAND_COMBINATION] = piece_stand_combination;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIECE_STAND_TURN] = piece_stand_turn[turn];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_PAWN] = rook_pawn;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PAWN_DROP] = pawn_drop;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIECE_STAND_Y] = piece_stand_y;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KNIGHT_CHECK] = knight_check;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PAWN_ADVANCE] = pawn_advance;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PAWN_PTYPEO] = pawn_ptypeo;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PROMOTED_MINOR_PIECE] = promoted_minor_piece;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING_PIECE_RELATIVE_NOSUPPORT] = nosupport;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::NON_PAWN_ATTACKED] = non_pawn_attacked[turn];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::NON_PAWN_ATTACKED_PTYPE] = non_pawn_attacked_ptype[turn];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPE_YY] = ptype_yy;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING3PIECES] = king3pieces;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_HEAD] = bishop_head;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KNIGHT_HEAD] = knight_head;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_PROMOTE_DEFENSE] = rook_promote_defense;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPE_COUNT] = ptype_count_value;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::LANCE_EFFECT_PIECE] = lance_effect_piece;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPE_Y_PAWN_Y] = ptype_y_pawn_y;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_AND_KING] = bishop_and_king;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIECE_FORK_TURN] = piece_fork_turn[turn];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_SILVER_KNIGHT] = rook_silver_knight;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_SILVER_KNIGHT] = bishop_silver_knight;
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT_SUPPORTED] =
+    King25EffectSupported::eval(black_attack_piece,
+                                white_attack_piece,
+                                black_attack_supported_piece,
+                                white_attack_supported_piece) +
+    King25EffectSupportedY::eval(black_attack_piece,
+                                 white_attack_piece,
+                                 black_attack_supported_piece,
+                                 white_attack_supported_piece,
+                                 state.kingSquare<BLACK>().y(),
+                                 state.kingSquare<WHITE>().y());
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING_ROOK_BISHOP] =
+    king_rook_bishop[BLACK] - king_rook_bishop[WHITE];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING_X_BLOCKED3] =
+    KingXBlocked3::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::GOLD_RETREAT] =
+    GoldFeatures::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::SILVER_RETREAT] =
+    SilverFeatures::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ALL_GOLD] =
+    AllGold::eval(black_gold_count);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ALL_MAJOR] =
+    AllMajor::eval(black_major_count);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT_DEFENSE] =
+    King25EffectDefense::eval(state, black_defense_effect, black_defense_piece,
+			      white_defense_effect, white_defense_piece) +
+    King25EffectYDefense::eval(state,
+			       black_defense_effect,
+			       black_defense_piece,
+			       white_defense_effect, white_defense_piece);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ANAGUMA_EMPTY] =
+    AnagumaEmpty::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::NO_PAWN_ON_STAND] =
+    NoPawnOnStand::eval(state, black_pawn_count);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::NON_PAWN_PIECE_STAND] =
+    NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE]);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PIN_PTYPE_ALL] =
+    PinPtypeAll::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING_MOBILITY] =
+    KingMobility::eval(state) + KingMobilitySum::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::GOLD_AND_SILVER_NEAR_KING] =
+    GoldAndSilverNearKing::eval(state,
+                                gs_near_king_count);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::PTYPE_COMBINATION] =
+    PtypeCombination::eval(ptypeo_mask);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_BOTH_SIDE] =
+    king25_both_side[BLACK] - king25_both_side[WHITE];
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_MOBILITY] =
+    King25Mobility::eval(state,
+                         black_king_vertical,
+                         white_king_vertical);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_STAND_FILE5] =
+    BishopStandFile5::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::MAJOR_CHECK_WITH_CAPTURE] =
+    MajorCheckWithCapture::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::SILVER_ADVANCE26] =
+    SilverAdvance26::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT3] =
+    King25Effect3::eval(state, effect25);                                 
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::BISHOP_BISHOP_PIECE] =
+    BishopBishopPiece::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_ROOK] =
+    RookRook::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ROOK_ROOK_PIECE] =
+    RookRookPiece::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::KING25_EFFECT_COUNT_COMBINATION] =
+    King25EffectCountCombination::eval(state, effect25);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::NON_PAWN_ATTACKED_PTYPE_PAIR] =
+    NonPawnAttackedPtypePair::eval(state);
+  debug_info.stage_values[OpenMidEndingEvalDebugInfo::ATTACK_MAJORS_IN_BASE] =
+    AttackMajorsInBase::eval(state);
+
   return debug_info;
 }
 
