@@ -612,6 +612,31 @@ void osl::progress::ml::NewProgress::updateSub(
   updateMain<P>(new_state, last_move);
 }
 
+osl::progress::ml::NewProgressDebugInfo osl::progress::ml::NewProgress::debugInfo() const
+{
+  NewProgressDebugInfo info;
+
+  info.black_values[NewProgressDebugInfo::ATTACK_5X3] = progresses[0];
+  info.black_values[NewProgressDebugInfo::DEFENSE_5X3] = defenses[0];
+  info.black_values[NewProgressDebugInfo::ATTACK5X5] = attack5x5_progresses[0];
+  info.black_values[NewProgressDebugInfo::STAND] = stand_progresses[0];
+  info.black_values[NewProgressDebugInfo::EFFECT5X5] = effect_progresses[0];
+  info.black_values[NewProgressDebugInfo::KING_RELATIVE_ATTACK] = king_relative_attack[0];
+  info.black_values[NewProgressDebugInfo::KING_RELATIVE_DEFENSE] = king_relative_defense[0];
+  info.black_values[NewProgressDebugInfo::NON_PAWN_ATTACKED_PAIR] = non_pawn_ptype_attacked_pair[0];
+
+  info.white_values[NewProgressDebugInfo::ATTACK_5X3] = progresses[1];
+  info.white_values[NewProgressDebugInfo::DEFENSE_5X3] = defenses[1];
+  info.white_values[NewProgressDebugInfo::ATTACK5X5] = attack5x5_progresses[1];
+  info.white_values[NewProgressDebugInfo::STAND] = stand_progresses[1];
+  info.white_values[NewProgressDebugInfo::EFFECT5X5] = effect_progresses[1];
+  info.white_values[NewProgressDebugInfo::KING_RELATIVE_ATTACK] = king_relative_attack[1];
+  info.white_values[NewProgressDebugInfo::KING_RELATIVE_DEFENSE] = king_relative_defense[1];
+  info.white_values[NewProgressDebugInfo::NON_PAWN_ATTACKED_PAIR] = non_pawn_ptype_attacked_pair[1];
+
+  return info;
+}
+
 namespace osl
 {
   namespace progress
