@@ -6,6 +6,7 @@
 #include <stdexcept>
 #include <cassert>
 #include <string>
+#include <sstream>
 
 /* ------------------------------------------------------------------------- */
 
@@ -181,6 +182,18 @@ show(Piece piece)
     + show(piece.ptype());
 }
 
+const std::string osl::record::csa::
+show(const Move *first, const Move *last)
+{
+  std::ostringstream out;
+  for (; first != last; ++first) {
+    if (first->isInvalid())
+      break;
+    out << show(*first);
+  }
+  return out.str();
+}
+
 /* ------------------------------------------------------------------------- */
 
 std::ostream& osl::csaShow(std::ostream& os,const Square pos)
