@@ -5,6 +5,10 @@
 #include "osl/record/kanjiCode.h"
 #include "osl/misc/sjis2euc.h"
 #include "osl/state/simpleState.h"
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/replace.hpp>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/foreach.hpp>
 #include <iostream>
 #include <fstream>
 #include <stdexcept>
@@ -17,10 +21,89 @@ namespace osl
   namespace record
   {
     void kakinokiParseLine(boost::shared_ptr<RecordVisitor>& rv,
-			   std::string s)
+			   std::string s, CArray<bool,9>& board_parsed)
     {
+      static const KanjiMove& Kanji_Move = KanjiMove::instance();
+      static const CArray<std::string,11> n_str = {{
+	  "", K_K1, K_K2, K_K3, K_K4, K_K5, K_K6, K_K7, K_K8, K_K9, K_K10
+	}};
       Record *record=rv->getRecord();
       SimpleState* state=rv->getState();
+      // header
+      if (s[0] == '|') {
+	if (s.size() < 1+3*9+1+2)
+	  throw KakinokiIOError("board too short in kakinokiParseLine "+s);
+	const int y = std::find(n_str.begin(), n_str.end(), s.substr(s.size()-2))
+	  - n_str.begin();
+	if (! (1 <= y && y <= 9))
+	  throw KakinokiIOError("unknown y in kakinokiParseLine "+s);
+	board_parsed[y-1] = true;
+	for (unsigned int x=9,i=1;i<s.length()&&x>0;i+=3,x--) {
+	  std::pair<Player,Ptype> pp=kakinoki::strToPiece(s.substr(i,3));
+	  if (! isPiece(pp.second))
+	    continue;
+	  state->setPiece(pp.first, Square(x,y), pp.second);
+	}
+      }
+      if (s.find(K_TESUU "--") == 0) {
+	// moves start
+	if (std::find(board_parsed.begin(), board_parsed.end(), true)
+	    == board_parsed.end()) {
+	  state->init(HIRATE);
+	  board_parsed.fill(true);
+	}
+	if (*std::min_element(board_parsed.begin(), board_parsed.end()) == false)
+	  throw KakinokiIOError("incomplete position description in kakinokiParseLine");
+	state->initPawnMask();
+	record->setInitialState(*state);
+	return;
+      }
+      if (s.size() > 6)
+      {
+	if (s == K_BLACK K_COLON) {
+	  record->setPlayer(BLACK, s.substr(6));
+	  return;
+	}
+	if (s == K_WHITE K_COLON) {
+	  record->setPlayer(WHITE, s.substr(6));
+	  return;
+	}
+        if (s.substr(0,6) == (K_KISEN K_COLON))
+        {
+          record->setTounamentName(s.substr(6));
+          return;
+        }
+	if (s.find(K_MOCHIGOMA K_COLON) != s.npos
+	    && s.find(K_NASHI) == s.npos) {
+	  std::string piece_str = s.substr(s.find(K_COLON)+2);
+	  boost::algorithm::replace_all(piece_str, K_SPACE, " ");
+	  std::vector<std::string> pieces;
+	  boost::algorithm::split(pieces, piece_str,
+				  boost::algorithm::is_any_of(" "));
+	  Player player;
+	  if (s.find(K_BLACK) == 0) player = BLACK;
+	  else if (s.find(K_WHITE) == 0) player = WHITE;
+	  else throw KakinokiIOError("error in stand "+ s);
+
+	  BOOST_FOREACH(const std::string& e, pieces) {
+	    if (e.empty()) continue;
+	    if (e.size() < 2) throw KakinokiIOError("error in stand "+ e);
+	    const Ptype ptype = Kanji_Move.toPtype(e.substr(0,2));
+	    int n = 1;
+	    if (e.size() >= 4)
+	      n = std::find(n_str.begin(),n_str.end(),e.substr(2,2))
+		- n_str.begin();
+	    if (e.size() >= 6)
+	      n = n * ((e.substr(2,2) == K_K10) ? 1 : 10)
+		+ (std::find(n_str.begin(),n_str.end(),e.substr(4,2))
+		   - n_str.begin());
+	    for (int i=0; i<n; ++i)
+	      state->setPiece(player, Square::STAND(), ptype);
+	  }
+	}
+      }
+
+      // moves
       if (s[0] == '*') 
       {
 	MoveRecord *mr = rv->getLastMove();
@@ -28,22 +111,6 @@ namespace osl
 	  mr->addComment(s.substr(1));
 	return;
       }
-      if (s.size() > 6)
-      {
-	if (equal(K_BLACK.begin(), K_BLACK.end(), s.begin())) {
-	  record->setPlayer(BLACK, s.substr(6));
-	  return;
-	}
-	if (equal(K_WHITE.begin(), K_WHITE.end(), s.begin())) {
-	  record->setPlayer(WHITE, s.substr(6));
-	  return;
-	}
-        if (s.substr(0,6) == (K_KISEN+K_COLON))
-        {
-          record->setTounamentName(s.substr(6));
-          return;
-        }
-      }
       if (s[0] != ' ') 
       {
 	if (rv->getLastMove() == 0)
@@ -83,10 +150,21 @@ namespace osl
   }
 }
 
+std::pair<osl::Player,osl::Ptype> osl::record::
+kakinoki::strToPiece(const std::string& s)
+{
+  static const KanjiMove& Kanji_Move = KanjiMove::instance();
+  if (s.size() != 3 || (s[0] != 'v' && s[0] != ' '))
+    throw KakinokiIOError("error in strToPiece " + s);
+  const Player pl = s[0] == ' ' ? BLACK : WHITE;
+  const Ptype ptype = Kanji_Move.toPtype(s.substr(1,2));
+  return std::make_pair(pl, ptype);
+}
+
 osl::Move osl::record::
 kakinoki::strToMove(const std::string& s, const SimpleState& state, Move last_move)
 {
-  static KanjiMove Kanji_Move;
+  static const KanjiMove& Kanji_Move = KanjiMove::instance();
   std::istringstream is(s);
   int move_number, from_number;
   std::string move_string;
@@ -164,10 +242,11 @@ void osl::record::kakinoki::
 InputStream::load(Record* rec)
 {
   //  rec->init();
-  state.init(HIRATE);
+  state.init();
   rv->setState(&state);
   rv->setRecord(rec);
   std::string line;
+  CArray<bool, 9> board_parsed = {{ false }};
   while (std::getline(is, line)) 
   {
     // quick hack for \r
@@ -185,7 +264,7 @@ InputStream::load(Record* rec)
 	&& line.find("separator") != line.npos)
       break;			// tanase shogi
     
-    kakinokiParseLine(rv, line);
+    kakinokiParseLine(rv, line, board_parsed);
   }
   assert(state.isConsistent());
 }
