@@ -13,6 +13,8 @@
 #include <iostream>
 using namespace boost::lambda;
 
+namespace 
+{
 int moveFromX(const osl::Move& move)
 {
   const osl::Square& p = move.from();
@@ -218,6 +220,7 @@ struct RemoveMoveFromYEqual :
     return p.y() == y;
   }
 };
+} // anonymous namespace
 
 osl::record::
 KanjiMove::KanjiMove()
@@ -227,26 +230,31 @@ KanjiMove::KanjiMove()
   {
     for (size_t y=1; y<=9; ++y)
     {
-      const std::string str = osl::record::StandardCharacters::suji[x] + 
-                              osl::record::StandardCharacters::dan[y];
-      str2position[str] = osl::Square(x,y);
+      const std::string str = StandardCharacters::suji[x] + 
+                              StandardCharacters::dan[y];
+      str2position[str] = Square(x,y);
     }
   }
-  str2piece[K_PAWN]      = osl::PAWN;
-  str2piece[K_PPAWN]     = osl::PPAWN;
-  str2piece[K_LANCE]     = osl::LANCE;
-  str2piece[K_PLANCE_D]  = osl::PLANCE;
-  str2piece[K_KNIGHT]    = osl::KNIGHT;
-  str2piece[K_PKNIGHT_D] = osl::PKNIGHT;
-  str2piece[K_SILVER]    = osl::SILVER;
-  str2piece[K_PSILVER_D] = osl::PSILVER;
-  str2piece[K_GOLD]      = osl::GOLD;
-  str2piece[K_BISHOP]    = osl::BISHOP;
-  str2piece[K_PBISHOP]   = osl::PBISHOP;
-  str2piece[K_ROOK]      = osl::ROOK;
-  str2piece[K_PROOK]     = osl::PROOK;
-  str2piece[K_PROOK2]    = osl::PROOK;
-  str2piece[K_KING]      = osl::KING;
+  str2piece[K_PAWN]      = PAWN;
+  str2piece[K_PPAWN]     = PPAWN;
+  str2piece[K_LANCE]     = LANCE;
+  str2piece[K_PLANCE_D]  = PLANCE;
+  str2piece[K_KNIGHT]    = KNIGHT;
+  str2piece[K_PKNIGHT_D] = PKNIGHT;
+  str2piece[K_SILVER]    = SILVER;
+  str2piece[K_PSILVER_D] = PSILVER;
+  str2piece[K_GOLD]      = GOLD;
+  str2piece[K_BISHOP]    = BISHOP;
+  str2piece[K_PBISHOP]   = PBISHOP;
+  str2piece[K_ROOK]      = ROOK;
+  str2piece[K_PROOK]     = PROOK;
+  str2piece[K_PROOK2]    = PROOK;
+  str2piece[K_KING]      = KING;
+
+  // pieces in kakinoki-style board specification
+  str2piece[K_PLANCE]  = PLANCE;
+  str2piece[K_PKNIGHT] = PKNIGHT;
+  str2piece[K_PSILVER] = PSILVER;
 }
 
 osl::record::
@@ -281,62 +289,62 @@ KanjiMove::selectCandidates(found_moves_t& found,
   assert(!str.empty());
   assert(found.size() >= 2);
 
-  if ( (str.substr(0,2) == K_MIGI && player == osl::BLACK) ||
-       (str.substr(0,2) == K_HIDARI && player == osl::WHITE) )
+  if ( (str.substr(0,2) == K_MIGI && player == BLACK) ||
+       (str.substr(0,2) == K_HIDARI && player == WHITE) )
   {
     found.sort( bind(moveFromX, boost::lambda::_1) < bind(moveFromX, boost::lambda::_2) );
     const osl::Move min = found.front();
     found.remove_if( RemoveMoveFromXOver(min.from().x()) ); // list really removes
   } 
-  else if ( (str.substr(0,2) == K_HIDARI && player == osl::BLACK) || 
-            (str.substr(0,2) == K_MIGI   && player == osl::WHITE) )
+  else if ( (str.substr(0,2) == K_HIDARI && player == BLACK) || 
+            (str.substr(0,2) == K_MIGI   && player == WHITE) )
   {
     found.sort( bind(moveFromX, boost::lambda::_1) < bind(moveFromX, boost::lambda::_2) );
-    const osl::Move max = found.back();
+    const Move max = found.back();
     found.remove_if( RemoveMoveFromXUnder(max.from().x()) ); // list really removes
   }
-  else if ( (str.substr(0,2) == K_SHITA && player == osl::BLACK) || 
-            (str.substr(0,2) == K_UE    && player == osl::WHITE) )
+  else if ( (str.substr(0,2) == K_SHITA && player == BLACK) || 
+            (str.substr(0,2) == K_UE    && player == WHITE) )
   {
     found.sort( bind(moveFromY, boost::lambda::_1) < bind(moveFromY, boost::lambda::_2) );
-    const osl::Move min = found.front();
+    const Move min = found.front();
     found.remove_if( RemoveMoveFromYOver(min.from().y()) ); // list really removes
   }
-  else if ( (str.substr(0,2) == K_UE    && player == osl::BLACK) || 
-            (str.substr(0,2) == K_SHITA && player == osl::WHITE) )
+  else if ( (str.substr(0,2) == K_UE    && player == BLACK) || 
+            (str.substr(0,2) == K_SHITA && player == WHITE) )
   {
     found.sort( bind(moveFromY, boost::lambda::_1) > bind(moveFromY, boost::lambda::_2) );
-    const osl::Move max = found.front();
+    const Move max = found.front();
     found.remove_if( RemoveMoveFromYUnder(max.from().y()) ); // list really removes
   }
   else if (str.substr(0,2) == K_YORU)
   {
     found.remove_if( std::not1(RemoveMoveFromYEqual(to_pos.y())) ); // list really removes
   }
-  else if (str.substr(0,2) == K_SUGU && player == osl::WHITE)
+  else if (str.substr(0,2) == K_SUGU && player == WHITE)
   {
     found.remove_if( std::not1(RemoveMoveFromXEqual(to_pos.x())) ); // or
     found.remove_if( std::not1(RemoveMoveFromYEqual(to_pos.y()-1)) ); // list really removes
   }
-  else if (str.substr(0,2) == K_SUGU && player == osl::BLACK)
+  else if (str.substr(0,2) == K_SUGU && player == BLACK)
 
   {
     found.remove_if( std::not1(RemoveMoveFromXEqual(to_pos.x())) ); // or
     found.remove_if( std::not1(RemoveMoveFromYEqual(to_pos.y()+1)) ); // list really removes
   }
-  else if (str.substr(0,2) == K_HIKU && player == osl::BLACK)
+  else if (str.substr(0,2) == K_HIKU && player == BLACK)
   {
     found.remove_if( RemoveMoveFromYGTE(to_pos.y()) ); // list really removes
   }
-  else if (str.substr(0,2) == K_HIKU && player == osl::WHITE)
+  else if (str.substr(0,2) == K_HIKU && player == WHITE)
   {
     found.remove_if( RemoveMoveFromYLTE(to_pos.y()) ); // list really removes
   }   
-  else if (str.substr(0,2) == K_YUKU && player == osl::BLACK)
+  else if (str.substr(0,2) == K_YUKU && player == BLACK)
   {
     found.remove_if( RemoveMoveFromYLTE(to_pos.y()) ); // list really removes
   }
-  else if (str.substr(0,2) == K_YUKU && player == osl::WHITE)
+  else if (str.substr(0,2) == K_YUKU && player == WHITE)
   {
     found.remove_if( RemoveMoveFromYGTE(to_pos.y()) ); // list really removes
   }
@@ -362,7 +370,7 @@ KanjiMove::strToMove(const std::string& orig,
 {
   std::string str(orig);
   assert(orig.size() >= 4*2 || (str.substr(2,2) == K_ONAZI && orig.size() >= 3*2));
-  const osl::Player player = str.substr(0,2) == K_BLACK_SIGN ? osl::BLACK : osl::WHITE;
+  const Player player = str.substr(0,2) == K_BLACK_SIGN ? BLACK : WHITE;
   assert(player == state.turn());
   str.erase(0,2);
 
@@ -479,6 +487,13 @@ KanjiMove::strToMove(const std::string& orig,
   return found.front();
 }
 
+const osl::record::KanjiMove& osl::record::
+KanjiMove::instance() 
+{
+  static const KanjiMove Kanji_Move;
+  return Kanji_Move;
+}
+
 // ;;; Local Variables:
 // ;;; mode:c++
 // ;;; c-basic-offset:2
