@@ -127,7 +127,11 @@ namespace record
   {
     namespace bf = boost::filesystem;
     const bf::path ipxfilename = bf::change_extension(bf::path(filename), ".ipx");
+#if BOOST_FILESYSTEM_VERSION >= 3
+    return ipxfilename.string();
+#else
     return ipxfilename.file_string();
+#endif
   }
 
   KisenIpxFile::KisenIpxFile(const std::string& fileName) 
