@@ -84,7 +84,13 @@ showXP(Move m)
 const osl::Move osl::record::psn::
 strToMove(const std::string& str, const SimpleState& s)
 {
-  assert(str.size() >= 4);
+  if (str.size() < 4)
+    throw ParseError("Invalid move string: " + str);
+  if (str == "pass")
+  {
+    return Move::PASS(s.turn());
+  }
+
   const Square to = strToPos(str.substr(2,2));
   if (str[1] == '*')
   {
@@ -111,6 +117,8 @@ strToPos(const std::string& str)
   assert(str.size() == 2);
   const int x = str[0] - '0';
   const int y = str[1] - 'a' + 1;
+  if (x <= 0 || x > 9 || y <= 0 || y > 9)
+    throw ParseError("Invalid square character: " + str);
   return Square(x, y);
 }
 
