@@ -1317,6 +1317,24 @@ stableThreshold(Player P, int last_value)
   return last_value - width - eval::delta(P);
 }
 
+namespace osl
+{
+  namespace
+  {
+    void find_threatmate(const SimpleHashTable& table, HashKey key,
+			 const search::SearchState2::PVVector& pv,
+			 CArray<bool, search::SearchState2::MaxDepth>& threatmate)
+    {
+      for (size_t i=0; i<pv.size(); ++i) {
+	key = key.newMakeMove(pv[i]);
+	const SimpleHashRecord *record = table.find(key);
+	threatmate[i] = record
+	  && record->threatmate().isThreatmate(key.turn());
+      }
+    }
+  }
+}
+
 template <class EvalT>
 void osl::search::AlphaBeta2Tree<EvalT>::
 updateRootPV(Player P, std::ostream& os, int result, Move m)
@@ -1346,11 +1364,14 @@ updateRootPV(Player P, std::ostream& os, int result, Move m)
   if (this->hasMonitor() && !this->prediction_for_speculative_search) {
     const double scale = OslConfig::usiOutputPawnValue()*2.0
       / eval_t::captureValue(newPtypeO(alt(P),PAWN));
+    CArray<bool, MaxDepth> threatmate = {{ 0 }};
+    find_threatmate(*this->table, currentHash(), pv[0], threatmate);
     BOOST_FOREACH(const boost::shared_ptr<SearchMonitor>& monitor,
 		  this->monitors())
       monitor->showPV(root_limit/200, this->recorder.allNodeCount(),
-		     this->elapsed(), static_cast<int>(result*scale), 
-		     m, &*pv[0].begin(), &*pv[0].end());
+		      this->elapsed(), static_cast<int>(result*scale), 
+		      m, &*pv[0].begin(), &*pv[0].end(),
+		      &threatmate[0], &threatmate[0]+pv[0].size());
   }
 #endif
   if (this->table->isVerbose()) {
@@ -1370,11 +1391,14 @@ addMultiPV(Player P, int result, Move m)
   if (this->hasMonitor() && !this->prediction_for_speculative_search) {
     const double scale = OslConfig::usiOutputPawnValue()*2.0
       / eval_t::captureValue(newPtypeO(alt(P),PAWN));
+    CArray<bool, MaxDepth> threatmate = {{ 0 }};
+    find_threatmate(*this->table, currentHash(), pv[0], threatmate);
     BOOST_FOREACH(const boost::shared_ptr<SearchMonitor>& monitor,
 		  this->monitors())
       monitor->showPV(root_limit/200, this->recorder.allNodeCount(),
-		     this->elapsed(), static_cast<int>(result*scale), 
-		     m, &*pv[0].begin(), &*pv[0].end());
+		      this->elapsed(), static_cast<int>(result*scale), 
+		      m, &*pv[0].begin(), &*pv[0].end(),
+		      &threatmate[0], &threatmate[0]+pv[0].size());
   }
 
   if (this->table->isVerbose()) {
@@ -1425,7 +1449,7 @@ showPV(std::ostream& os, int result, Move m, char stable_char) const
 	= this->table->find(HashKey(state));
       if (record && 
 	  record->threatmate().isThreatmate(state.turn()))
-	str += "(" + record::K_TSUMERO + ")";
+	str += "(" K_TSUMERO ")";
     }
     std::string converted = IconvConvert::eucToLang(str);
     if (! converted.empty())
@@ -1627,12 +1651,6 @@ AlphaBeta2(const NumEffectState& s, checkmate_t& c,
   : AlphaBeta2Tree<EvalT>(s, c, t, r)
 {
   MoveGenerator::initOnce();
-#ifndef GPSONE
-  if (OslConfig::usiMode()) {
-    boost::shared_ptr<SearchMonitor> monitor(new UsiMonitor);
-    this->addMonitor(monitor);
-  }
-#endif
 }
 
 template <class EvalT>
@@ -1729,21 +1747,6 @@ int osl::search::AlphaBeta2<EvalT>::
 alphaBetaSearchRoot(MoveLogProb& best_move, int limit)
 {
   const Player Turn = this->state().turn();
-  if (OslConfig::forceRootWindow()) {
-    const double scale = this->eval.captureValue(newPtypeO(WHITE,PAWN))/2;
-    std::pair<int,int> ab = OslConfig::rootWindow();
-    ab = std::make_pair((int)(ab.first*scale/OslConfig::usiOutputPawnValue()),
-			(int)(ab.second*scale/OslConfig::usiOutputPawnValue()));
-    if (ab.first % 2 == 0) 
-      ab.first += (ab.first > 0) ? 1 : -1;
-    if (ab.second % 2 == 0) 
-      ab.second += (ab.second > 0) ? 1 : -1;
-    if (eval::betterThan(Turn, ab.first, ab.second))
-      ab = std::make_pair(ab.second, ab.first);
-    Window root_window(Turn, ab.first, ab.second);
-    assert(root_window.isConsistent());
-    return alphaBetaSearchRoot(root_window, best_move, limit);
-  }
   Window root_window = this->fullWindow(Turn);
   return alphaBetaSearchRoot(root_window, best_move, limit);
 }
@@ -1770,18 +1773,6 @@ computeBestMoveIteratively(int limit, const int step,
 	      << " iteration " << this->nextIterationCoefficient()
 	      << " mem " << std::fixed << std::setprecision(2)
 	      << OslConfig::memoryUseRatio()*100.0 << "%";
-    if (OslConfig::forceRootWindow()) {
-      std::pair<int,int> ab = OslConfig::rootWindow();
-      if (ab.first == ab.second)
-	std::cerr << " root-null-window "
-		  << (ab.first*100.0/OslConfig::usiOutputPawnValue());
-      else
-	std::cerr << " root-window ["
-		  << (ab.first*100.0/OslConfig::usiOutputPawnValue())
-		  << ", "
-		  << (ab.second*100.0/OslConfig::usiOutputPawnValue())
-		  << "]";
-    }
     std::cerr << "\n";
   }
   initial_limit = std::min(initial_limit, limit);
@@ -2123,7 +2114,7 @@ alphaBetaSearchRoot(Window window, MoveLogProb& best_move, int limit)
 		    this->monitors())
 	monitor->showPV(1, this->recorder.allNodeCount(),
 			this->elapsed(), static_cast<int>(result*scale), 
-			best_move.move(), 0, 0);
+			best_move.move(), 0, 0, 0, 0);
     }
 #endif
     return result;
@@ -2392,6 +2383,14 @@ alphaBetaSearchRoot(Window window, MoveLogProb& best_move, int limit)
 	assert(best_move.move() == this->shared_root->last_pv.back().pv[0]);
       }
     }
+#ifndef GPSONE
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      BOOST_FOREACH(const boost::shared_ptr<SearchMonitor>& monitor,
+		    this->monitors())
+	monitor->depthFinishedNormally(limit/200);
+    }
+#endif
   } catch (std::runtime_error& e) {
     if (this->table->isVerbose())
       std::cerr << e.what() << "\n";
@@ -2433,13 +2432,12 @@ alphaBetaSearchRoot(Window window, MoveLogProb& best_move, int limit)
   {
     const double scale = OslConfig::usiOutputPawnValue()*2.0
       / this->eval.captureValue(newPtypeO(alt(P),PAWN));
-    const int value = OslConfig::forceRootWindow() 
-      ? ValueNone : this->winByCheckmate(alt(P));
+    const int value = this->winByCheckmate(alt(P));
     BOOST_FOREACH(const boost::shared_ptr<SearchMonitor>& monitor,
 		  this->monitors())
       monitor->showPV(limit/200, this->recorder.allNodeCount(),
 		      this->elapsed(), static_cast<int>(value*scale), 
-		      Move::INVALID(), 0, 0);
+		      Move::INVALID(), 0, 0, 0, 0);
   }
 #endif
   return best_value;
