@@ -6,6 +6,7 @@
 #include "osl/search/simpleHashTable.h"
 #include "osl/search/usiReporter.h"
 #include "osl/misc/nonBlockDelete.h"
+#include <boost/foreach.hpp>
 #include <iostream>
 #include <bitset>
 #ifdef _WIN32
@@ -685,9 +686,12 @@ examineMovesRootPar(int tree_id)
        m.validMove() && ! stopping();
        m = shared->nextMove(tree_id).first) {
 #ifndef GPSONE
-    if (OslConfig::usiMode()) {
-      boost::mutex::scoped_lock lk(OslConfig::lock_io);      
-      UsiReporter::rootMove(m.move());
+    if (this->elapsed() > 1.0)
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      BOOST_FOREACH(const boost::shared_ptr<SearchMonitor>& monitor,
+		    this->monitors())
+	monitor->rootMove(m.move());
     }
 #endif
     try {
