@@ -9,8 +9,8 @@ void osl::search::
 HistoryTable::extractTopN(Player p, vector<OutputEntry>& out, size_t limit) const
 {
   out.clear();
-  for (size_t i=0; i<=Position(9,9).uintValue(); ++i)
-    for (size_t j=Position(1,1).uintValue(); j<=Position(9,9).uintValue(); ++j)
+  for (size_t i=0; i<=Square(9,9).uintValue(); ++i)
+    for (size_t j=Square(1,1).uintValue(); j<=Square(9,9).uintValue(); ++j)
       if (table[p][i][j].value > 100)
 	out.push_back(OutputEntry(i, j, table[p][i][j].value));
   std::sort(out.begin(), out.end(), std::greater<OutputEntry>());
@@ -24,7 +24,7 @@ std::ostream& osl::search::operator<<(std::ostream& os, const HistoryTable::Outp
   if (e.from_or_ptype < PTYPE_SIZE)
     os << Ptype(e.from_or_ptype);
   else
-    os << Position::makeDirect(e.from_or_ptype);
+    os << Square::makeDirect(e.from_or_ptype);
   return os << " => " << e.to << " " << e.value << ")";
 }
 
