@@ -11,7 +11,7 @@ void osl::search::SearchMonitor::newDepth(int)
 {
 }
 
-void osl::search::SearchMonitor::showPV(int, size_t, double, int, Move, const Move *, const Move *)
+void osl::search::SearchMonitor::showPV(int, size_t, double, int, Move, const Move *, const Move *, const bool*, const bool*)
 {
 }
 
@@ -40,6 +40,10 @@ void osl::search::SearchMonitor::rootLossByCheckmate()
 {
 }
 
+void osl::search::SearchMonitor::depthFinishedNormally(int)
+{
+}
+
 // ;;; Local Variables:
 // ;;; mode:c++
 // ;;; c-basic-offset:2
