@@ -46,7 +46,11 @@ static void convert(const std::string &kisen_filename,
   {
     const boost::filesystem::path ipx_path =
       boost::filesystem::change_extension(boost::filesystem::path(kisen_filename), ".ipx");
+#if BOOST_FILESYSTEM_VERSION >= 3
+    const std::string ipx = ipx_path.string();
+#else
     const std::string ipx = ipx_path.file_string();
+#endif
     ipx_ofs.reset(new std::ofstream(ipx.c_str()));
     ipx_writer.reset(new osl::record::KisenIpxWriter(*ipx_ofs));
   }
